import pytest

import mbedtls
import mbedtls.hkdf as _hkdf
import mbedtls.hmac as _hmac


@pytest.mark.skipif(
    not mbedtls.has_feature("HKDF"),
    reason="requires HKDF support in libmbedtls",
)
class TestHKDF:
    @pytest.fixture(params=[0, 128])
    def key(self, request, randbytes):
        return randbytes(request.param)

    @pytest.fixture(
        params=[
            _hmac.md2,
            _hmac.md4,
            _hmac.md5,
            _hmac.sha1,
            _hmac.sha224,
            _hmac.sha256,
            _hmac.sha384,
            _hmac.sha512,
            _hmac.ripemd160,
        ]
    )
    def hmac(self, request, key):
        feature = request.param.__name__
        if not mbedtls.has_feature(
            {"sha224": "sha256", "sha384": "sha512"}.get(feature, feature)
        ):
            return pytest.skip("requires %s support in mbedtls" % feature)
        return request.param

    @pytest.fixture
    def salt(self, hmac, key, randbytes):
        return randbytes(hmac(key).digest_size)

    @pytest.fixture(params=[16, 128, 500, 1000])
    def info(self, request, randbytes):
        return randbytes(request.param)

    @pytest.fixture(params=[16, 200, 500])
    def length(self, request):
        return request.param

    def test_hkdf(self, key):
        okm = _hkdf.hkdf(key, 32, b"")
        assert len(okm) == 32

    def test_hkdf_with_options(self, key, salt, info, hmac, length):
        okm = _hkdf.hkdf(key, length, info, salt, hmac)
        assert len(okm) == length

    def test_extract_and_expand(self, key):
        prk = _hkdf.extract(key)
        assert len(prk) == _hmac.sha256(b"").digest_size

        okm = _hkdf.expand(prk, 32, b"")
        assert len(okm) == 32

    def test_extract_and_expand_with_options(
        self, hmac, key, salt, info, length
    ):
        prk = _hkdf.extract(key, salt, hmac)
        assert len(prk) == hmac(b"").digest_size

        okm = _hkdf.expand(prk, length, info, hmac)
        assert len(okm) == length


def to_bytes(number, size):
    return number.to_bytes(size, byteorder="big")


class TestHKDF_RFC5869_TestVectors:
    def test_case_1(self):
        algorithm = _hmac.sha256
        ikm = to_bytes(0x0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B, 22)
        salt = to_bytes(0x000102030405060708090A0B0C, 13)
        info = to_bytes(0xF0F1F2F3F4F5F6F7F8F9, 10)
        length = 42
        prk = to_bytes(
            int(
                "0x077709362c2e32df0ddc3f0dc47bba63"
                "90b6c73bb50f9c3122ec844ad7c2b3e5",
                0,
            ),
            32,
        )
        okm = to_bytes(
            int(
                "0x3cb25f25faacd57a90434f64d0362f2a"
                "2d2d0a90cf1a5a4c5db02d56ecc4c5bf"
                "34007208d5b887185865",
                0,
            ),
            42,
        )

        assert _hkdf.extract(ikm, salt, algorithm) == prk
        assert _hkdf.expand(prk, length, info, algorithm) == okm
        assert _hkdf.hkdf(ikm, length, info, salt, algorithm) == okm

    def test_case_2(self):
        algorithm = _hmac.sha256
        ikm = to_bytes(
            int(
                "0x000102030405060708090a0b0c0d0e0f"
                "101112131415161718191a1b1c1d1e1f"
                "202122232425262728292a2b2c2d2e2f"
                "303132333435363738393a3b3c3d3e3f"
                "404142434445464748494a4b4c4d4e4f",
                0,
            ),
            80,
        )
        salt = to_bytes(
            int(
                "0x606162636465666768696a6b6c6d6e6f"
                "707172737475767778797a7b7c7d7e7f"
                "808182838485868788898a8b8c8d8e8f"
                "909192939495969798999a9b9c9d9e9f"
                "a0a1a2a3a4a5a6a7a8a9aaabacadaeaf",
                0,
            ),
            80,
        )
        info = to_bytes(
            int(
                "0xb0b1b2b3b4b5b6b7b8b9babbbcbdbebf"
                "c0c1c2c3c4c5c6c7c8c9cacbcccdcecf"
                "d0d1d2d3d4d5d6d7d8d9dadbdcdddedf"
                "e0e1e2e3e4e5e6e7e8e9eaebecedeeef"
                "f0f1f2f3f4f5f6f7f8f9fafbfcfdfeff",
                0,
            ),
            80,
        )
        length = 82
        prk = to_bytes(
            int(
                "0x06a6b88c5853361a06104c9ceb35b45c"
                "ef760014904671014a193f40c15fc244",
                0,
            ),
            32,
        )
        okm = to_bytes(
            int(
                "0xb11e398dc80327a1c8e7f78c596a4934"
                "4f012eda2d4efad8a050cc4c19afa97c"
                "59045a99cac7827271cb41c65e590e09"
                "da3275600c2f09b8367793a9aca3db71"
                "cc30c58179ec3e87c14c01d5c1f3434f"
                "1d87",
                0,
            ),
            82,
        )

        assert _hkdf.extract(ikm, salt, algorithm) == prk
        assert _hkdf.expand(prk, length, info, algorithm) == okm
        assert _hkdf.hkdf(ikm, length, info, salt, algorithm) == okm

    def test_case_3(self):
        algorithm = _hmac.sha256
        ikm = to_bytes(0x0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B, 22)
        salt = b""
        info = b""
        length = 42
        prk = to_bytes(
            int(
                "0x19ef24a32c717b167f33a91d6f648bdf"
                "96596776afdb6377ac434c1c293ccb04",
                0,
            ),
            32,
        )
        okm = to_bytes(
            int(
                "0x8da4e775a563c18f715f802a063c5a31"
                "b8a11f5c5ee1879ec3454e5f3c738d2d"
                "9d201395faa4b61a96c8",
                0,
            ),
            42,
        )

        assert _hkdf.extract(ikm, salt, algorithm) == prk
        assert _hkdf.expand(prk, length, info, algorithm) == okm
        assert _hkdf.hkdf(ikm, length, info, salt, algorithm) == okm

    def test_case_4(self):
        algorithm = _hmac.sha1
        ikm = to_bytes(0x0B0B0B0B0B0B0B0B0B0B0B, 11)
        salt = to_bytes(0x000102030405060708090A0B0C, 13)
        info = to_bytes(0xF0F1F2F3F4F5F6F7F8F9, 10)
        length = 42
        prk = to_bytes(0x9B6C18C432A7BF8F0E71C8EB88F4B30BAA2BA243, 20)
        okm = to_bytes(
            int(
                "0x085a01ea1b10f36933068b56efa5ad81"
                "a4f14b822f5b091568a9cdd4f155fda2"
                "c22e422478d305f3f896",
                0,
            ),
            42,
        )

        assert _hkdf.extract(ikm, salt, algorithm) == prk
        assert _hkdf.expand(prk, length, info, algorithm) == okm
        assert _hkdf.hkdf(ikm, length, info, salt, algorithm) == okm

    def test_case_5(self):
        algorithm = _hmac.sha1
        ikm = to_bytes(
            int(
                "0x000102030405060708090a0b0c0d0e0f"
                "101112131415161718191a1b1c1d1e1f"
                "202122232425262728292a2b2c2d2e2f"
                "303132333435363738393a3b3c3d3e3f"
                "404142434445464748494a4b4c4d4e4f",
                0,
            ),
            80,
        )
        salt = to_bytes(
            int(
                "0x606162636465666768696a6b6c6d6e6f"
                "707172737475767778797a7b7c7d7e7f"
                "808182838485868788898a8b8c8d8e8f"
                "909192939495969798999a9b9c9d9e9f"
                "a0a1a2a3a4a5a6a7a8a9aaabacadaeaf",
                0,
            ),
            80,
        )
        info = to_bytes(
            int(
                "0xb0b1b2b3b4b5b6b7b8b9babbbcbdbebf"
                "c0c1c2c3c4c5c6c7c8c9cacbcccdcecf"
                "d0d1d2d3d4d5d6d7d8d9dadbdcdddedf"
                "e0e1e2e3e4e5e6e7e8e9eaebecedeeef"
                "f0f1f2f3f4f5f6f7f8f9fafbfcfdfeff",
                0,
            ),
            80,
        )
        length = 82
        prk = to_bytes(0x8ADAE09A2A307059478D309B26C4115A224CFAF6, 20)
        okm = to_bytes(
            int(
                "0x0bd770a74d1160f7c9f12cd5912a06eb"
                "ff6adcae899d92191fe4305673ba2ffe"
                "8fa3f1a4e5ad79f3f334b3b202b2173c"
                "486ea37ce3d397ed034c7f9dfeb15c5e"
                "927336d0441f4c4300e2cff0d0900b52"
                "d3b4",
                0,
            ),
            82,
        )

        assert _hkdf.extract(ikm, salt, algorithm) == prk
        assert _hkdf.expand(prk, length, info, algorithm) == okm
        assert _hkdf.hkdf(ikm, length, info, salt, algorithm) == okm

    def test_case_6(self):
        algorithm = _hmac.sha1
        ikm = to_bytes(0x0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B, 22)
        salt = b""
        info = b""
        length = 42
        prk = to_bytes(0xDA8C8A73C7FA77288EC6F5E7C297786AA0D32D01, 20)
        okm = to_bytes(
            int(
                "0x0ac1af7002b3d761d1e55298da9d0506"
                "b9ae52057220a306e07b6b87e8df21d0"
                "ea00033de03984d34918",
                0,
            ),
            42,
        )

        assert _hkdf.extract(ikm, salt, algorithm) == prk
        assert _hkdf.expand(prk, length, info, algorithm) == okm
        assert _hkdf.hkdf(ikm, length, info, salt, algorithm) == okm

    def test_case_7(self):
        algorithm = _hmac.sha1
        ikm = to_bytes(0x0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C, 22)
        info = b""
        length = 42
        prk = to_bytes(0x2ADCCADA18779E7C2077AD2EB19D3F3E731385DD, 20)
        okm = to_bytes(
            int(
                "0x2c91117204d745f3500d636a62f64f0a"
                "b3bae548aa53d423b0d1f27ebba6f5e5"
                "673a081d70cce7acfc48",
                0,
            ),
            42,
        )

        assert _hkdf.extract(ikm, digestmod=algorithm) == prk
        assert _hkdf.expand(prk, length, info, algorithm) == okm
        assert _hkdf.hkdf(ikm, length, info, digestmod=algorithm) == okm



# SPDX-License-Identifier: MIT
# Copyright (c) 2019, Mathias Laurin

"""The ARIA algorithm is a symmetric block cipher that can encrypt and
decrypt information. It is defined by the Korean Agency for Technology
and Standards (KATS) in *KS X 1213:2004* (in Korean, but see
http://210.104.33.10/ARIA/index-e.html in English) and also described by
the IETF in *RFC 5794*.

"""


from mbedtls.exceptions import TLSError

from . import _cipher

__all__ = ["block_size", "key_size", "new"]


block_size = 16
key_size = None


def new(key, mode, iv=None):
    """Return a `Cipher` object that can perform ARIA encryption and
    decryption.

    ARIA is a block cipher designed in 2003 by a large group of South
    Korean researchers. In 2004, the Korean Agency for Technology and
    Standards selected it as a standard cryptographic technique.

    Parameters:
        key (bytes): The key to encrypt decrypt.
        mode (int): The mode of operation of the cipher.
        iv (bytes or None): The initialization vector (IV).  The IV is
            required for every mode but ECB and CTR where it is ignored.
            If not set, the IV is initialized to all 0, which should not
            be used for encryption.

    """
    mode = _cipher.Mode(mode)
    if mode in {
        _cipher.Mode.ECB,
        _cipher.Mode.CBC,
        # _cipher.Mode.CFB128,
        _cipher.Mode.CTR,
        _cipher.Mode.GCM,
    }:
        if len(key) * 8 not in {128, 192, 256}:
            raise TLSError(
                msg="key size must 16, 24, or 32 bytes, got %i" % len(key)
            )
    else:
        raise TLSError(msg="unsupported mode %r" % mode)
    name = ("ARIA-%i-%s" % (len(key) * 8, mode.name)).encode("ascii")
    return _cipher.Cipher(name, key, mode, iv)



# SPDX-License-Identifier: MIT
# Copyright (c) 2019, Mathias Laurin

"""Platform utils."""


cimport mbedtls._platform as _plt


__all__ = ["zeroize"]


def zeroize(char[:] buffer not None):
    _plt.mbedtls_platform_zeroize(&buffer[0], buffer.size)


def __self_test():
    binary = bytearray(b"0123456789abcdef")
    cdef unsigned char[:] c_binary = binary
    zeroize(c_binary)
    assert binary == b"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0", binary

    string = bytearray(b"0123456789abcdef")
    cdef char[:] c_string = string
    zeroize(c_string)
    assert string == b"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0", string



# SPDX-License-Identifier: MIT
# Copyright (c) 2015, Elaborated Networks GmbH
# Copyright (c) 2019, Mathias Laurin

"""Generic message digest wrapper (hash algorithm)."""


import mbedtls._md as _md

algorithms_guaranteed = _md.algorithms_guaranteed
algorithms_available = _md.algorithms_available


Hash = _md.Hash


def new(name, buffer=None):
    """A generic constructor that takes the string name of the desired
    algorithm as its first parameter.

    """
    return Hash(name, buffer)


def md2(buffer=None):
    """MD2 message-digest algorithm."""
    return Hash("md2", buffer)


def md4(buffer=None):
    """MD4 message-digest algorithm."""
    return Hash("md4", buffer)


def md5(buffer=None):
    """MD5 message-digest algorithm."""
    return Hash("md5", buffer)


def sha1(buffer=None):
    """Secure Hash Algorithm 1 (SHA-1)."""
    return Hash("sha1", buffer)


def sha224(buffer=None):
    """Secure Hash Algorithm 2 (SHA-2) with 224 bits hash value."""
    return Hash("sha224", buffer)


def sha256(buffer=None):
    """Secure Hash Algorithm 2 (SHA-2) with 256 bits hash value."""
    return Hash("sha256", buffer)


def sha384(buffer=None):
    """Secure Hash Algorithm 2 (SHA-2) with 384 bits hash value."""
    return Hash("sha384", buffer)


def sha512(buffer=None):
    """Secure Hash Algorithm 2 (SHA-2) with 512 bits hash value."""
    return Hash("sha512", buffer)


def ripemd160(buffer=None):
    """RACE Integrity Primitives Evaluation Message Digest (RIPEMD) with
    160 bits hash value."""
    return Hash("ripemd160", buffer)



"""Unit tests for mbedtls.cipher."""

# Disable checks for violations that are acceptable in tests.
# pylint: disable=missing-docstring
# pylint: disable=attribute-defined-outside-init
# pylint: disable=invalid-name, redefined-outer-name

import pickle

import pytest

import mbedtls
import mbedtls.cipher as mb
from mbedtls.cipher._cipher import CIPHER_NAME, Cipher, get_supported_ciphers
from mbedtls.exceptions import *


def test_cipher_list():
    assert len(CIPHER_NAME) == 74


def test_get_supported_ciphers():
    cl = get_supported_ciphers()
    assert cl and set(cl).issubset(set(CIPHER_NAME))


def test_wrong_size_raises_exception():
    with pytest.raises(NotImplementedError):
        Cipher(b"AES-512-ECB", b"", mb.Mode.ECB, b"")


def test_random_name_raises_exception():
    with pytest.raises(NotImplementedError):
        Cipher(b"RANDOM TEXT IS NOT A CIPHER", b"", mb.Mode.ECB, b"")


def test_zero_length_raises_exception():
    with pytest.raises(NotImplementedError):
        Cipher(b"", b"", mb.Mode.ECB, b"")


@pytest.mark.parametrize("mode", [mb.MODE_CBC, mb.Mode.CBC])
def test_cbc_raises_value_error_without_iv(mode):
    with pytest.raises(ValueError):
        Cipher(b"AES-512-CBC", b"", mode, b"")


@pytest.mark.parametrize("mode", [mb.MODE_CFB, mb.Mode.CFB])
def test_cfb_raises_value_error_without_iv(mode):
    with pytest.raises(ValueError):
        Cipher(b"AES-512-CFB", b"", mode, b"")


def _mode(mode):
    return (
        mode
        if mbedtls.has_feature("cipher_mode_%s" % mode.name)
        else pytest.skip("requires %s support in libmbedtls" % mode.name)
    )


class _TestCipher:
    @pytest.fixture(
        params=[
            mb.Mode.ECB,
            _mode(mb.Mode.CBC),
            _mode(mb.Mode.CFB),
            _mode(mb.Mode.CTR),
            mb.Mode.GCM,
            mb.Mode.CCM,
        ]
    )
    def mode(self, request):
        return request.param

    @pytest.fixture(params=[])
    def unsupported_mode(self, request):
        return request.param

    @pytest.fixture
    def iv_size(self, mode):
        raise NotImplementedError

    @pytest.fixture
    def iv(self, iv_size, randbytes):
        return randbytes(iv_size)

    @pytest.fixture
    def key_size(self):
        raise NotImplementedError

    @pytest.fixture(params=[])
    def invalid_key_size(self, request):
        return request.param

    @pytest.fixture
    def module(self):
        raise NotImplementedError

    @pytest.fixture
    def cipher(self):
        raise NotImplementedError

    @pytest.fixture
    def key(self, key_size, randbytes):
        return randbytes(key_size)

    @pytest.fixture
    def invalid_key(self, invalid_key_size, randbytes):
        return randbytes(invalid_key_size)

    @pytest.fixture
    def cipher(self, module, key, mode, iv):
        try:
            return module.new(key, mode, iv)
        except NotImplementedError:
            return pytest.skip("unsupported cipher")

    @pytest.fixture
    def data(self, cipher, mode, randbytes):
        # `block_size` is limited for ECB because it is a block cipher.
        assert cipher.block_size
        return randbytes(
            cipher.block_size
            if mode is mb.Mode.ECB
            else cipher.block_size * 128
        )

    def test_pickle(self, cipher):
        with pytest.raises(TypeError) as excinfo:
            pickle.dumps(cipher)

        assert str(excinfo.value).startswith("cannot pickle")

    def test_mode_accessor(self, cipher, mode):
        assert cipher.mode is mode

    def test_iv_size_accessor(self, cipher, iv_size):
        assert cipher.iv_size == iv_size

    def test_key_size_accessor(self, cipher, key_size):
        assert cipher.key_size == key_size

    def test_name_accessor(self, cipher):
        assert cipher.name in CIPHER_NAME

    def test_str(self, cipher):
        assert str(cipher) == cipher.name.decode("ascii")

    def test_type_accessor(self, cipher):
        assert CIPHER_NAME[cipher._type] == cipher.name

    def test_unsupported_mode(self, module, key, unsupported_mode, iv):
        with pytest.raises(TLSError):
            module.new(key, unsupported_mode, iv)

    def test_invalid_key_size(self, module, invalid_key, mode, iv):
        with pytest.raises(TLSError):
            module.new(invalid_key, mode, iv)

    def test_encrypt_decrypt(self, cipher, data):
        assert cipher.decrypt(cipher.encrypt(data)) == data

    def test_encrypt_nothing_raises(self, cipher):
        with pytest.raises(TLSError):
            cipher.encrypt(b"")

    def test_decrypt_nothing_raises(self, cipher):
        with pytest.raises(TLSError):
            cipher.decrypt(b"")

    def test_module_level_block_size(self, module, cipher):
        assert module.block_size == cipher.block_size

    def test_module_level_key_size(self, module, cipher):
        assert module.key_size in {module.key_size, None}


class _TestAEADCipher(_TestCipher):
    @pytest.fixture
    def cipher(self, module, key, mode, iv, ad):
        return module.new(key, mode, iv, ad)

    @pytest.fixture(params=[0, 1, 16, 256])
    def ad(self, mode, randbytes, request):
        return randbytes(request.param)

    def test_cbc_requires_padding(self, mode, cipher, data):
        data += b"\0"
        if mode is mb.Mode.CBC:
            with pytest.raises(ValueError):
                cipher.encrypt(data)
        assert cipher.decrypt(*cipher.encrypt(data)) == data

    def test_encrypt_decrypt(self, cipher, data):
        msg, tag = cipher.encrypt(data)
        assert cipher.decrypt(msg, tag) == data

    def test_decrypt_nothing_raises(self, cipher, data):
        msg, tag = cipher.encrypt(data)
        with pytest.raises(TLSError):
            cipher.decrypt(b"", tag)


@pytest.mark.skipif(
    not mbedtls.has_feature("aes"), reason="requires AES support in libmbedtls"
)
class _TestAESBase(_TestCipher):
    @pytest.fixture(params=[mb.Mode.STREAM, mb.Mode.CHACHAPOLY])
    def unsupported_mode(self, request):
        return request.param

    @pytest.fixture(params=[8, 15, 128])
    def invalid_key_size(self, request):
        return request.param

    @pytest.fixture
    def module(self):
        return mb.AES


class TestAES(_TestAESBase):
    @pytest.fixture(
        params=[
            mb.Mode.ECB,
            _mode(mb.Mode.CBC),
            _mode(mb.Mode.CFB),
            _mode(mb.Mode.CTR),
            _mode(mb.Mode.OFB),
        ]
    )
    def mode(self, request):
        return request.param

    @pytest.fixture
    def iv_size(self, mode):
        return {mb.Mode.ECB: 0}.get(mode, 16)

    @pytest.fixture(params=[16, 24, 32])
    def key_size(self, request):
        return request.param


class TestAES_XTS(_TestAESBase):
    @pytest.fixture(params=[_mode(mb.Mode.XTS)])
    def mode(self, request):
        return request.param

    @pytest.fixture
    def iv_size(self, mode):
        return {mb.Mode.ECB: 0}.get(mode, 16)

    @pytest.fixture(params=[32, 64])
    def key_size(self, request):
        return request.param


class TestAES_AEAD(_TestAEADCipher):
    @pytest.fixture(params=[mb.Mode.GCM, mb.Mode.CCM])
    def mode(self, request):
        return request.param

    @pytest.fixture(params=[8, 15, 128])
    def invalid_key_size(self, request):
        return request.param

    @pytest.fixture
    def iv_size(self, mode):
        return {mb.Mode.ECB: 0}.get(mode, 12)

    @pytest.fixture(params=[16, 24, 32])
    def key_size(self, request):
        return request.param

    @pytest.fixture
    def module(self):
        return mb.AES


@pytest.mark.skipif(
    not mbedtls.has_feature("arc4"),
    reason="requires ARC4 support in libmbedtls",
)
class TestARC4(_TestCipher):
    @pytest.fixture(params=[mb.Mode.STREAM])
    def mode(self, request):
        return request.param

    @pytest.fixture
    def iv_size(self, mode):
        return {mb.Mode.ECB: 0}.get(mode, 0)

    @pytest.fixture(params=[16])
    def key_size(self, request):
        return request.param

    @pytest.fixture(params=[8, 15, 32])
    def invalid_key_size(self, request):
        return request.param

    @pytest.fixture
    def module(self):
        return mb.ARC4


@pytest.mark.skipif(
    not mbedtls.has_feature("aria"),
    reason="requires Aria support in libmbedtls",
)
class TestARIA(_TestCipher):
    @pytest.fixture(
        params=[
            mb.Mode.ECB,
            _mode(mb.Mode.CBC),
            _mode(mb.Mode.CTR),
            mb.Mode.GCM,
        ]
    )
    def mode(self, request):
        return request.param

    @pytest.fixture(
        params=[
            mb.Mode.CFB,
            mb.Mode.OFB,
            mb.Mode.STREAM,
            mb.Mode.CCM,
            mb.Mode.XTS,
            mb.Mode.CHACHAPOLY,
        ]
    )
    def unsupported_mode(self, request):
        return request.param

    @pytest.fixture
    def iv_size(self, mode):
        return {mb.Mode.ECB: 0, mb.Mode.GCM: 12}.get(mode, 16)

    @pytest.fixture(params=[16, 24, 32])
    def key_size(self, request):
        return request.param

    @pytest.fixture(params=[8, 15, 64])
    def invalid_key_size(self, request):
        return request.param

    @pytest.fixture
    def module(self):
        return mb.ARIA


@pytest.mark.skipif(
    not mbedtls.has_feature("blowfish"),
    reason="requires Blowfish support in libmbedtls",
)
class TestBlowfish(_TestCipher):
    @pytest.fixture(
        params=[
            mb.Mode.ECB,
            _mode(mb.Mode.CBC),
            _mode(mb.Mode.CFB),
            _mode(mb.Mode.CTR),
        ]
    )
    def mode(self, request):
        return request.param

    @pytest.fixture(
        params=[
            mb.Mode.OFB,
            mb.Mode.GCM,
            mb.Mode.STREAM,
            mb.Mode.CCM,
            mb.Mode.XTS,
            mb.Mode.CHACHAPOLY,
        ]
    )
    def unsupported_mode(self, request):
        return request.param

    @pytest.fixture
    def iv_size(self, mode):
        return {mb.Mode.ECB: 0}.get(mode, 8)

    @pytest.fixture(params=range(4, 57))
    def key_size(self, request):
        return request.param

    @pytest.fixture(params=[3, 57])
    def invalid_key_size(self, request):
        return request.param

    @pytest.fixture
    def module(self):
        return mb.Blowfish

    @pytest.mark.skip("Blowfish always returns key_size == 16")
    def test_key_size_accessor(self, cipher, key_size):
        assert cipher.key_size == 16


@pytest.mark.skipif(
    not mbedtls.has_feature("camellia"),
    reason="requires Camellia support in libmbedtls",
)
class TestCamellia(_TestCipher):
    @pytest.fixture(
        params=[
            mb.Mode.ECB,
            _mode(mb.Mode.CBC),
            _mode(mb.Mode.CFB),
            _mode(mb.Mode.CTR),
            mb.Mode.GCM,
        ]
    )
    def mode(self, request):
        return request.param

    @pytest.fixture(
        params=[
            mb.Mode.OFB,
            mb.Mode.STREAM,
            mb.Mode.CCM,
            mb.Mode.XTS,
            mb.Mode.CHACHAPOLY,
        ]
    )
    def unsupported_mode(self, request):
        return request.param

    @pytest.fixture
    def iv_size(self, mode):
        return {mb.Mode.ECB: 0, mb.Mode.GCM: 12}.get(mode, 16)

    @pytest.fixture(params=[16, 24, 32])
    def key_size(self, request):
        return request.param

    @pytest.fixture(params=[8, 15, 64])
    def invalid_key_size(self, request):
        return request.param

    @pytest.fixture
    def module(self):
        return mb.Camellia


@pytest.mark.skipif(
    not mbedtls.has_feature("des"), reason="requires DES support in libmbedtls"
)
class _TestDESBase(_TestCipher):
    @pytest.fixture(params=[mb.Mode.ECB, _mode(mb.Mode.CBC)])
    def mode(self, request):
        return request.param

    @pytest.fixture(
        params=[
            mb.Mode.CFB,
            mb.Mode.OFB,
            mb.Mode.CTR,
            mb.Mode.GCM,
            mb.Mode.STREAM,
            mb.Mode.CCM,
            mb.Mode.XTS,
            mb.Mode.CHACHAPOLY,
        ]
    )
    def unsupported_mode(self, request):
        return request.param

    @pytest.fixture
    def iv_size(self, mode):
        return {mb.Mode.ECB: 0}.get(mode, 8)

    @pytest.fixture(params=[4, 64])
    def invalid_key_size(self, request):
        return request.param


class TestDES(_TestDESBase):
    @pytest.fixture(params=[8])
    def key_size(self, request):
        return request.param

    @pytest.fixture
    def module(self):
        return mb.DES


class TestDES3(_TestDESBase):
    @pytest.fixture(params=[24])
    def key_size(self, request):
        return request.param

    @pytest.fixture
    def module(self):
        return mb.DES3


class TestDES3dbl(_TestDESBase):
    @pytest.fixture(params=[16])
    def key_size(self, request):
        return request.param

    @pytest.fixture
    def module(self):
        return mb.DES3dbl


@pytest.mark.skipif(
    not mbedtls.has_feature("chacha20"),
    reason="requires CHACHA20 support in libmbedtls",
)
class TestCHACHA20(_TestCipher):
    @pytest.fixture(params=[mb.Mode.STREAM])
    def mode(self, request):
        return request.param

    @pytest.fixture(
        params=[
            mb.Mode.ECB,
            mb.Mode.CBC,
            mb.Mode.CFB,
            mb.Mode.OFB,
            mb.Mode.CTR,
            mb.Mode.GCM,
            mb.Mode.CCM,
            mb.Mode.XTS,
        ]
    )
    def unsupported_mode(self, request):
        return request.param

    @pytest.fixture
    def iv_size(self, mode):
        return {mb.Mode.ECB: 0}.get(mode, 12)

    @pytest.fixture(params=[32])
    def key_size(self, request):
        return request.param

    @pytest.fixture(params=[8, 16, 64])
    def invalid_key_size(self, request):
        return request.param

    @pytest.fixture
    def module(self):
        return mb.CHACHA20


@pytest.mark.skipif(
    not all(
        (mbedtls.has_feature("chacha20"), mbedtls.has_feature("chachapoly"))
    ),
    reason="requires CHACHA20 support in libmbedtls",
)
class TestCHACHA20AEAD(_TestAEADCipher):
    @pytest.fixture(params=[mb.Mode.CHACHAPOLY])
    def mode(self, request):
        return request.param

    @pytest.fixture(
        params=[
            mb.Mode.ECB,
            mb.Mode.CBC,
            mb.Mode.CFB,
            mb.Mode.OFB,
            mb.Mode.CTR,
            mb.Mode.GCM,
            mb.Mode.CCM,
            mb.Mode.XTS,
        ]
    )
    def unsupported_mode(self, request):
        return request.param

    @pytest.fixture
    def iv_size(self, mode):
        return {mb.Mode.ECB: 0}.get(mode, 12)

    @pytest.fixture(params=[32])
    def key_size(self, request):
        return request.param

    @pytest.fixture(params=[8, 16, 64])
    def invalid_key_size(self, request):
        return request.param

    @pytest.fixture
    def module(self):
        return mb.CHACHA20



#!/usr/bin/env python
"""An example DTLS/TLS server.

Run ./programs/server.py --help.

"""

import argparse
import socket
import time
from contextlib import suppress
from functools import partial

from mbedtls._tls import _enable_debug_output, _set_debug_level
from mbedtls.tls import (
    DTLSConfiguration,
    HelloVerifyRequest,
    ServerContext,
    TLSConfiguration,
)

__all__ = ["Server"]


def _make_tls_connection(sock):
    assert sock
    conn, _addr = sock.accept()
    conn.do_handshake()
    return conn


def _make_dtls_connection(sock):
    assert sock
    conn, addr = sock.accept()
    conn.setcookieparam(addr[0].encode("ascii"))
    with suppress(HelloVerifyRequest):
        conn.do_handshake()

    _, (conn, addr) = conn, conn.accept()
    _.close()
    conn.setcookieparam(addr[0].encode("ascii"))
    conn.do_handshake()
    return conn


class Server:
    def __init__(self, srv_conf, proto, address):
        super().__init__()
        self.srv_conf = srv_conf
        self.proto = proto
        self.address = address
        self._make_connection = {
            socket.SOCK_STREAM: _make_tls_connection,
            socket.SOCK_DGRAM: _make_dtls_connection,
        }[self.proto]
        self._sock = None

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, *exc_info):
        self.stop()

    def __del__(self):
        self.stop()

    @property
    def context(self):
        if self._sock is None:
            return None
        return self._sock.context

    def start(self):
        if self._sock:
            self.stop()

        self._sock = ServerContext(self.srv_conf).wrap_socket(
            socket.socket(socket.AF_INET, self.proto)
        )
        self._sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self._sock.bind(self.address)
        if self.proto is socket.SOCK_STREAM:
            self._sock.listen(1)

    def stop(self):
        if not self._sock:
            return

        self._sock.close()
        self._sock = None

    def run(self, conn_handler):
        if not self._sock:
            raise ConnectionRefusedError("server not started")

        while True:
            self._run(conn_handler)

    def _run(self, conn_handler):
        with self._make_connection(self._sock) as conn:
            conn_handler(conn)


def echo_handler(conn, *, packet_size):
    while True:
        data = conn.recv(packet_size)
        if data:
            break
        # Avoid tight loop.
        time.sleep(0.01)
    sent = 0
    view = memoryview(data)
    while sent != len(data):
        sent += conn.send(view[sent:])


def parse_args():
    class PSKStoreArg(argparse.Action):
        def __call__(self, parser, namespace, values, option_string=None):
            def entry(kv):
                k, v = kv.split(b"=")
                return k.decode("utf-8"), v

            setattr(
                namespace,
                self.dest,
                dict(entry(kv) for kv in values.split(b",")),
            )

    parser = argparse.ArgumentParser(description="server")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        "--tls", dest="proto", action="store_const", const=socket.SOCK_STREAM
    )
    group.add_argument(
        "--dtls", dest="proto", action="store_const", const=socket.SOCK_DGRAM
    )
    parser.add_argument("--address", default="0.0.0.0")
    parser.add_argument("--port", default=4433, type=int)
    parser.add_argument("--debug", type=int)
    parser.add_argument(
        "--psk-store",
        type=lambda x: x.encode("latin1"),
        action=PSKStoreArg,
        metavar="CLI1=SECRET1,CLI2=SECRET2...",
    )
    return parser.parse_args()


def main(args):
    conf = {
        socket.SOCK_STREAM: TLSConfiguration,
        socket.SOCK_DGRAM: DTLSConfiguration,
    }[args.proto](
        pre_shared_key_store=args.psk_store,
        validate_certificates=False,
    )

    if args.debug is not None:
        _enable_debug_output(conf)
        _set_debug_level(args.debug)

    with Server(conf, args.proto, (args.address, args.port)) as srv:
        srv.run(partial(echo_handler, packet_size=4069))


if __name__ == "__main__":
    import faulthandler

    faulthandler.enable()
    with suppress(KeyboardInterrupt):
        main(parse_args())



"""Unit tests for mbedtls.random."""

import pickle
import random
from collections import defaultdict

import pytest

import mbedtls._random as _drbg
import mbedtls.mpi as _mpi
from mbedtls.exceptions import TLSError


def sample(start, end, k=20):
    return random.sample(range(start, end), k)


class TestEntropy:
    @pytest.fixture
    def entropy(self):
        return _drbg.Random()._entropy

    def test_pickle(self, entropy):
        with pytest.raises(TypeError) as excinfo:
            pickle.dumps(entropy)

        assert str(excinfo.value).startswith("cannot pickle")

    def test_gather(self, entropy):
        # Only test that this does not raise.
        entropy.gather()

    @pytest.mark.parametrize("length", range(64))
    def test_retrieve(self, entropy, length):
        assert len(entropy.retrieve(length)) == length

    @pytest.mark.parametrize("length", (100,))
    def test_retrieve_long_block_raises_exception(self, entropy, length):
        with pytest.raises(TLSError):
            entropy.retrieve(length)

    def test_update(self, entropy, randbytes):
        # Only test that this does not raise.
        buf = randbytes(64)
        entropy.update(buf)

    def test_not_reproducible(self, entropy):
        assert entropy.retrieve(8) != entropy.retrieve(8)

    def test_random_retrieve(self, entropy):
        size = 4
        number = entropy.retrieve(size)
        result = defaultdict(int)
        for _ in range(250):
            result[entropy.retrieve(size) == number] += 1
        assert result[True] <= 1


class TestRandom:
    @pytest.fixture
    def random(self):
        return _drbg.Random()

    def test_pickle(self, random):
        with pytest.raises(TypeError) as excinfo:
            pickle.dumps(random)

        assert str(excinfo.value).startswith("cannot pickle")

    def test_reseed(self, random):
        random._reseed()

    @pytest.mark.repeat(100)
    def test_random(self, random):
        value = random.random()
        assert isinstance(value, float)
        assert 0 <= value < 1

    @pytest.mark.repeat(100)
    def test_random_not_reproducible(self, random):
        assert random.random() != random.random()

    @pytest.mark.repeat(100)
    def test_getrandbits_not_reproducible(self, random):
        assert random.getrandbits(64) != random.getrandbits(64)

    @pytest.mark.repeat(100)
    def test_random_initial_values(self):
        random = _drbg.Random()
        other = _drbg.Random()
        assert random.random() != other.random()

    @pytest.mark.repeat(100)
    def test_getrandbits_initial_values(self):
        random = _drbg.Random()
        other = _drbg.Random()
        assert random.getrandbits(64) != other.getrandbits(64)

    @pytest.mark.parametrize("nbits", [1, 5, 10, 50, 100, 300, 500])
    def test_getrandbits_size(self, random, nbits):
        value = random.getrandbits(nbits)
        assert isinstance(value, _mpi.MPI)
        assert value.bit_length() == pytest.approx(nbits, abs=7)
        assert value.bit_length() <= nbits

    @pytest.mark.parametrize("nbits", [-1, 0])
    def test_getrandbits_negative_k_raises_value_error(self, random, nbits):
        with pytest.raises(ValueError):
            random.getrandbits(nbits)

    def test_getrandbits_nonint_k_raises_type_error(self, random):
        with pytest.raises(TypeError):
            random.getrandbits("a")



# SPDX-License-Identifier: MIT
# Copyright (c) 2016, Elaborated Networks GmbH
# Copyright (c) 2019, Mathias Laurin

"""Declarations for `mbedtls/ctr_drbg.h`."""


cdef extern from "mbedtls/entropy.h" nogil:
    ctypedef struct mbedtls_entropy_context:
        pass

    void mbedtls_entropy_init(mbedtls_entropy_context *ctx)
    void mbedtls_entropy_free(mbedtls_entropy_context *ctx)
    # int mbedtls_entropy_add_source(
    #     mbedtls_entropy_context *ctx,
    #     mbedtls_entropy_f_source_ptr f_source,
    #     void *p_source,
    #     size_t threshold, int strong)
    int mbedtls_entropy_gather(mbedtls_entropy_context *ctx)
    int mbedtls_entropy_func(void *data, unsigned char *output, size_t len)
    int mbedtls_entropy_update_manual(
        mbedtls_entropy_context *ctx,
        const unsigned char *data,
        size_t len)
    # int mbedtls_entropy_write_seed_file(
    #     mbedtls_entropy_context *ctx,
    #     const char *path)
    # int mbedtls_entropy_update_seed_file(
    #     mbedtls_entropy_context *ctx,
    #     const char *path)


cdef extern from "mbedtls/ctr_drbg.h" nogil:
    ctypedef struct mbedtls_ctr_drbg_context:
        pass

    void mbedtls_ctr_drbg_init(mbedtls_ctr_drbg_context *ctx)
    int mbedtls_ctr_drbg_seed(
        mbedtls_ctr_drbg_context *ctx,
        int (*f_entropy)(void *, unsigned char *, size_t),
        void *p_entropy,
        const unsigned char *custom,
        size_t len)
    void mbedtls_ctr_drbg_free(mbedtls_ctr_drbg_context *ctx)

    # void mbedtls_ctr_drbg_set_prediction_resistance(
    #     mbedtls_ctr_drbg_context *ctx,
    #     int resistance)
    # void mbedtls_ctr_drbg_set_entropy_len(
    #     mbedtls_ctr_drbg_context *ctx,
    #     size_t len)
    # void mbedtls_ctr_drbg_set_reseed_interval(
    #     mbedtls_ctr_drbg_context *ctx,
    #     int interval)

    int mbedtls_ctr_drbg_reseed(
        mbedtls_ctr_drbg_context *ctx,
        const unsigned char *additional, size_t len)
    void mbedtls_ctr_drbg_update(
        mbedtls_ctr_drbg_context *ctx,
        const unsigned char *additional, size_t add_len)
    # int mbedtls_ctr_drbg_random_with_add(
    #     void *p_rng,
    #     unsigned char *output, size_t output_len,
    #     const unsigned char *additional, size_t add_len)
    int mbedtls_ctr_drbg_random(
        void *p_rng,
        unsigned char *output, size_t output_len)
    # int mbedtls_ctr_drbg_write_seed_file(
    #     mbedtls_ctr_drbg_context *ctx, const char *path)
    # int mbedtls_ctr_drbg_update_seed_file(
    #     mbedtls_ctr_drbg_context *ctx, const char *path)


cdef class _Entropy:
    cdef mbedtls_entropy_context _ctx


cdef class Random:
    cdef mbedtls_ctr_drbg_context _ctx
    cdef _Entropy _entropy


# RNG for internal use only.
cpdef Random default_rng()



#!/usr/bin/env python
"""An example DTLS/TLS client.

Run ./programs/client.py --help.

"""

import argparse
import socket
import time
from contextlib import suppress

from mbedtls._tls import _enable_debug_output, _set_debug_level
from mbedtls.exceptions import TLSError
from mbedtls.tls import ClientContext, DTLSConfiguration, TLSConfiguration

__all__ = ["Client"]


def _echo_tls(sock, buffer, chunksize):
    view = memoryview(buffer)
    received = bytearray()
    for idx in range(0, len(view), chunksize):
        part = view[idx : idx + chunksize]
        _sent = sock.send(part)
        received += sock.recv(chunksize)
    return received


def _echo_dtls(sock, buffer, chunksize):
    view = memoryview(buffer)
    received = bytearray()
    while len(received) != len(buffer):
        part = view[len(received) : len(received) + chunksize]
        _sent = sock.send(part)
        data, _addr = sock.recvfrom(chunksize)
        received += data
        if not data:
            # Avoid tight loop.
            time.sleep(0.01)
    return received


class Client:
    def __init__(self, cli_conf, proto, srv_address, srv_hostname):
        super().__init__()
        self.cli_conf = cli_conf
        self.proto = proto
        self.srv_address = srv_address
        self.srv_hostname = srv_hostname
        self._sock = None
        self._echo = {
            socket.SOCK_STREAM: _echo_tls,
            socket.SOCK_DGRAM: _echo_dtls,
        }[self.proto]

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, *exc_info):
        self.stop()

    def __del__(self):
        self.stop()

    @property
    def context(self):
        if self._sock is None:
            return None
        return self._sock.context

    def do_handshake(self):
        if not self._sock:
            return

        self._sock.do_handshake()

    def echo(self, buffer, chunksize):
        if not self._sock:
            return b""

        return bytes(self._echo(self._sock, buffer, chunksize))

    def start(self):
        if self._sock:
            self.stop()

        self._sock = ClientContext(self.cli_conf).wrap_socket(
            socket.socket(socket.AF_INET, self.proto),
            server_hostname=self.srv_hostname,
        )
        self._sock.connect(self.srv_address)

    def stop(self):
        if not self._sock:
            return

        with suppress(TLSError, OSError):
            self._sock.close()
        self._sock = None

    def restart(self):
        self.stop()
        self.start()


def parse_args():
    class PSKArg(argparse.Action):
        def __call__(self, parser, namespace, values, option_string=None):
            def entry(kv):
                k, v = kv.split(b"=")
                return k.decode("utf-8"), v

            setattr(namespace, self.dest, entry(values))

    parser = argparse.ArgumentParser(description="client")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        "--tls", dest="proto", action="store_const", const=socket.SOCK_STREAM
    )
    group.add_argument(
        "--dtls", dest="proto", action="store_const", const=socket.SOCK_DGRAM
    )
    parser.add_argument("--address", default="127.0.0.1")
    parser.add_argument("--port", default=4433, type=int)
    parser.add_argument("--debug", type=int)
    parser.add_argument("--server-name", default="localhost")
    parser.add_argument(
        "--psk",
        type=lambda x: x.encode("latin1"),
        action=PSKArg,
        metavar="CLI=SECRET",
    )
    parser.add_argument("message", default="hello")
    return parser.parse_args()


def main(args):
    conf = {
        socket.SOCK_STREAM: TLSConfiguration,
        socket.SOCK_DGRAM: DTLSConfiguration,
    }[args.proto](pre_shared_key=args.psk, validate_certificates=False)

    if args.debug is not None:
        _enable_debug_output(conf)
        _set_debug_level(args.debug)

    with Client(
        conf, args.proto, (args.address, args.port), args.server_name
    ) as cli:
        cli.do_handshake()
        received = cli.echo(args.message.encode("utf-8"), 1024)
    print(received.decode("utf-8"))


if __name__ == "__main__":
    main(parse_args())



# SPDX-License-Identifier: MIT
# Copyright (c) 2015, Elaborated Networks GmbH
# Copyright (c) 2018, Mathias Laurin

"""python-mbedtls is a this wrapper to ARM's mbed TLS library."""


import mbedtls.cipher as cipher
import mbedtls.exceptions as exceptions
import mbedtls.hashlib as hashlib
import mbedtls.hkdf as hkdf
import mbedtls.hmac as hmac
import mbedtls.pk as pk
import mbedtls.secrets as secrets
import mbedtls.tls as tls
import mbedtls.version as version
import mbedtls.x509 as x509

__version__ = "2.0.1"

__all__ = (
    "cipher",
    "exceptions",
    "hash",
    "hashlib",
    "hkdf",
    "hmac",
    "pk",
    "secrets",
    "tls",
    "version",
    "x509",
)


has_feature = version.has_feature



# SPDX-License-Identifier: MIT
# Copyright (c) 2018, Mathias Laurin

"""A ring buffer.

See Also:
    https://github.com/dhess/c-ringbuf/

"""


cdef struct ring_buffer_ctx:
    unsigned char *buf
    unsigned char *head
    unsigned char *tail
    size_t _size


cdef c_init(ring_buffer_ctx *, size_t)
cdef void c_free(ring_buffer_ctx *) nogil
cdef void c_clear(ring_buffer_ctx *) nogil
cdef size_t c_capacity(ring_buffer_ctx *) nogil
cdef size_t c_len(ring_buffer_ctx *) nogil
cdef c_peek(ring_buffer_ctx *, size_t)
cdef size_t c_peekinto(ring_buffer_ctx *, unsigned char *, size_t) nogil
cdef c_read(ring_buffer_ctx *, size_t)
cdef size_t c_readinto(ring_buffer_ctx *, unsigned char *,  size_t) nogil
cdef size_t c_consume(ring_buffer_ctx *, size_t) nogil
cdef size_t c_write(ring_buffer_ctx *, const unsigned char *, size_t) nogil


cdef class RingBuffer:
    cdef ring_buffer_ctx _ctx



# SPDX-License-Identifier: MIT
# Copyright (c) 2015, Elaborated Networks GmbH
# Copyright (c) 2018, Mathias Laurin

"""Declarations from `mbedtls/md.h`."""


cdef extern from "mbedtls/md_internal.h" nogil:
    ctypedef struct mbedtls_md_info_t:
        int block_size


cdef extern from "mbedtls/md.h" nogil:
    ctypedef enum mbedtls_md_type_t:
        pass

    ctypedef struct mbedtls_md_context_t:
        const mbedtls_md_info_t *md_info

    const int *mbedtls_md_list()
    const mbedtls_md_info_t *mbedtls_md_info_from_string(const char *md_name)
    # mbedtls_md_info_from_type

    void mbedtls_md_init(mbedtls_md_context_t *ctx)
    void mbedtls_md_free(mbedtls_md_context_t *ctx)

    int mbedtls_md_setup(
        mbedtls_md_context_t *ctx,
        const mbedtls_md_info_t *md_info,
        int hmac)

    int mbedtls_md_clone(mbedtls_md_context_t *dst,
                         const mbedtls_md_context_t *src)
    unsigned char mbedtls_md_get_size(const mbedtls_md_info_t *md_info)
    mbedtls_md_type_t mbedtls_md_get_type(const mbedtls_md_info_t *md_info)
    const char *mbedtls_md_get_name(const mbedtls_md_info_t *md_info)

    int mbedtls_md_starts(mbedtls_md_context_t *ctx)
    int mbedtls_md_update(
        mbedtls_md_context_t *ctx,
        const unsigned char *input,
        size_t ilen)
    int mbedtls_md_finish(
        mbedtls_md_context_t *ctx,
        unsigned char *output)
    # mbedtls_md
    # mbedtls_md_file

    int mbedtls_md_hmac_starts(
        mbedtls_md_context_t *ctx,
        const unsigned char *key,
        size_t keylen)
    int mbedtls_md_hmac_update(
        mbedtls_md_context_t *ctx,
        const unsigned char *input,
        size_t ilen)
    int mbedtls_md_hmac_finish(
        mbedtls_md_context_t *ctx,
        unsigned char *output)
    int mbedtls_md_hmac_reset(mbedtls_md_context_t *ctx)
    # mbedtls_md_hmac


cdef class MDBase:
    cdef const mbedtls_md_info_t* _info
    cdef mbedtls_md_context_t _ctx
    cdef _finish(self, const unsigned char *output)


cdef class Hash(MDBase):
    pass


cdef class Hmac(MDBase):
    pass



"""Unit test mbedtls.exceptions."""

import pytest

from mbedtls.exceptions import TLSError, check_error


@pytest.mark.parametrize(
    "err, msg", ((0x003C, "ENTROPY"), (0x1080, "PEM"), (0x2200, "X509"))
)
def test_mbedtls_error(err, msg):
    with pytest.raises(TLSError, match=r"%s - .+" % msg) as exc:
        check_error(-err)


def test_other_error():
    with pytest.raises(TLSError, match="error message") as exc:
        raise TLSError(msg="error message")



import ctypes
import ctypes.util
import os
import platform
import re
import sys
from contextlib import suppress

from setuptools import Extension, find_packages, setup


def _get_version():
    pattern = re.compile(r'^__version__ = ["]([.\w]+?)["]')
    with open(
        os.path.join(
            os.path.dirname(__file__), "src", "mbedtls", "__init__.py"
        )
    ) as f:
        for line in f:
            match = pattern.match(line)
            if match:
                return match.group(1)
        raise RuntimeError()


VERSION = _get_version()
MBEDTLS_VERSION = "2.28.0"
DOWNLOAD_URL = "https://github.com/Synss/python-mbedtls/tarball/%s" % VERSION


__mbedtls_version_info__ = tuple(map(int, MBEDTLS_VERSION.split(".")))
__mbedtls_url__ = "https://tls.mbed.org"


if "--with-coverage" in sys.argv:
    sys.argv.remove("--with-coverage")
    COVERAGE = True
else:
    COVERAGE = False


setup_requires = [
    # Setuptools 18.0 properly handles Cython extensions.
    "setuptools >= 18.0",
    # Cython 0.28 handles const memoryviews.
    "cython >= 0.28.0",
]
install_requires = [
    "certifi",
    'typing_extensions; python_version < "3.8"',
]
tests_require = [
    "readme_renderer",
]


def mbedtls_version(lib):
    null = b"\0"
    output = 18 * null
    output_p = ctypes.c_char_p(output)
    lib.mbedtls_version_get_string_full(output_p)
    return output.strip(null).decode("ascii")


def mbedtls_version_info(lib):
    version = lib.mbedtls_version_get_number()
    return tuple(version >> shift & 0xFF for shift in (24, 16, 8))


def check_mbedtls_support(version, url):
    library = ctypes.util.find_library("mbedtls")
    if not library:
        sys.stderr.write("  Library not found{sep}".format(sep=os.linesep))
        sys.stderr.write(
            "  The paths are probably not set correctly but let's try anyway{sep}".format(
                sep=os.linesep
            )
        )
        return
    try:
        lib = ctypes.cdll.LoadLibrary(library)
        sys.stdout.write("  loading: {!r}\n".format(lib._name))
        sys.stdout.write(
            "  mbedtls version: {!s}{sep}".format(
                mbedtls_version(lib), sep=os.linesep
            )
        )
        sys.stdout.write("  python-mbedtls version: {0}\n".format(VERSION))
    except OSError as exc:
        lib = None
        sys.stderr.write("  {exc!s}{sep}".format(exc=exc, sep=os.linesep))
    if lib and mbedtls_version_info(lib) < version[:2]:
        message = (
            "  python-mbedtls requires at least mbedtls {major}.{minor}".format(
                major=version[0], minor=version[1]
            ),
            "  The latest version of mbedtls may be obtained from {url}.".format(
                url=url
            ),
            "",
        )
        sys.stderr.writelines(os.linesep.join(message))
        sys.exit(1)


def extensions(coverage=False):
    def from_env(var):
        with suppress(KeyError):
            return filter(None, os.environ[var].split(ENVSEP))
        return ()

    WINDOWS = platform.system() == "Windows"
    ENVSEP = ";" if WINDOWS else ":"

    libraries = (
        [
            "AdvAPI32",  # `Crypt*` calls from `library/entropy_poll.c`
            "mbedTLS",
        ]
        if WINDOWS
        else ["mbedcrypto", "mbedtls", "mbedx509"]
    )
    library_dirs = list(from_env("LIB" if WINDOWS else "LIBRARY_PATH"))

    for dirpath, _, filenames in os.walk("src"):
        for fn in filenames:
            root, ext = os.path.splitext(fn)
            if ext != ".pyx":
                continue
            mod = ".".join(dirpath.split(os.sep)[1:] + [root])
            extension = Extension(
                mod,
                sources=[os.path.join(dirpath, fn)],
                library_dirs=library_dirs,
                libraries=libraries,
                define_macros=[
                    ("CYTHON_TRACE", "1"),
                    ("CYTHON_TRACE_NOGIL", "1"),
                ]
                if coverage
                else [],
            )
            extension.cython_directives = {"language_level": "3str"}
            if coverage:
                extension.cython_directives["linetrace"] = True
            yield extension


def options(coverage=False):
    if coverage:
        return {}

    return {
        "build": {
            "build_base": os.sep.join(
                ("build", "%i.%i.%i" % sys.version_info[:3])
            )
        },
        "build_ext": {"cython_c_in_temp": True},
    }


def readme():
    with open("README.rst") as f:
        return f.read().replace(":math:", "")


if len(sys.argv) > 1 and any(
    (sys.argv[1].startswith("build"), sys.argv[1].startswith("bdist"))
):
    check_mbedtls_support(
        version=__mbedtls_version_info__, url=__mbedtls_url__
    )


setup(
    name="python-mbedtls",
    version=VERSION,
    description=(
        "hash, hmac, RSA, ECC, X.509, TLS, DTLS, handshakes, and secrets "
        "with an mbed TLS back end"
    ),
    long_description=readme(),
    long_description_content_type="text/x-rst",
    author="Mathias Laurin",
    author_email="Mathias.Laurin@github.com",
    license="MIT License",
    url="https://github.com/Synss/python-mbedtls",
    download_url=DOWNLOAD_URL,
    ext_modules=list(extensions(COVERAGE)),
    options=options(COVERAGE),
    package_dir={"": "src"},
    packages=find_packages("src"),
    setup_requires=setup_requires,
    install_requires=install_requires,
    tests_require=tests_require,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Cython",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Topic :: Security :: Cryptography",
    ],
)



import pytest

from mbedtls import version


def test_version_info():
    assert len(version.version_info) == 3
    assert version.version_info >= (2, 7, 9)


def test_version():
    assert version.version == "mbed TLS %i.%i.%i" % version.version_info


@pytest.mark.parametrize("feature", ["i do not exist"])
def test_feature_false(feature):
    assert version.has_feature(feature) is False


@pytest.mark.parametrize(
    "feature",
    (
        "md5",
        "md5_c",
        "mbedtls_md5",
        "mbedtls_md5_c",
        "MD5",
        "MD5_C",
        "MBEDTLS_MD5",
        "MBEDTLS_MD5_C",
    ),
)
def test_feature_true(feature):
    assert version.has_feature(feature) is True



# SPDX-License-Identifier: MIT
# Copyright (c) 2015, Elaborated Networks GmbH
# Copyright (c) 2019, Mathias Laurin

"""Declarations from `mbedtls/cipher.h`."""


cdef:
    enum: MAX_BLOCK_LENGTH = 16
    enum: MAX_IV_LENGTH = 16


cdef extern from "mbedtls/cipher.h" nogil:
    ctypedef enum mbedtls_cipher_padding_t:
        MBEDTLS_PADDING_PKCS7 = 0
        MBEDTLS_PADDING_ONE_AND_ZEROS
        MBEDTLS_PADDING_ZEROS_AND_LEN
        MBEDTLS_PADDING_ZEROS
        MBEDTLS_PADDING_NONE

    ctypedef enum mbedtls_operation_t:
        MBEDTLS_DECRYPT = 0, MBEDTLS_ENCRYPT = 1

    ctypedef enum mbedtls_cipher_type_t:
        pass

    ctypedef enum mbedtls_cipher_mode_t:
        # The numbering is standardized.
        MBEDTLS_MODE_NONE = 0,
        MBEDTLS_MODE_ECB,
        MBEDTLS_MODE_CBC,
        MBEDTLS_MODE_CFB,
        MBEDTLS_MODE_OFB,
        MBEDTLS_MODE_CTR,
        MBEDTLS_MODE_GCM,
        MBEDTLS_MODE_STREAM,
        MBEDTLS_MODE_CCM,
        MBEDTLS_MODE_XTS,
        MBEDTLS_MODE_CHACHAPOLY

    ctypedef struct mbedtls_cipher_base_t:
        pass

    ctypedef struct mbedtls_cipher_info_t:
        mbedtls_cipher_type_t type
        mbedtls_cipher_mode_t mode
        unsigned int key_bitlen
        const char* name
        unsigned int iv_size
        int flags
        unsigned int block_size
        const mbedtls_cipher_base_t *base

    ctypedef struct mbedtls_cipher_context_t:
        pass

    const int* mbedtls_cipher_list()
    const mbedtls_cipher_info_t* mbedtls_cipher_info_from_string(
        const char* cipher_name)
    const mbedtls_cipher_info_t* mbedtls_cipher_info_from_type(
        const mbedtls_cipher_type_t)
    # mbedtls_cipher_info_from_values

    void mbedtls_cipher_init(mbedtls_cipher_context_t* ctx)
    void mbedtls_cipher_free(mbedtls_cipher_context_t* ctx)

    int mbedtls_cipher_setup(
        mbedtls_cipher_context_t* ctx,
        const mbedtls_cipher_info_t* cipher_info)

    unsigned int mbedtls_cipher_get_block_size(
        const mbedtls_cipher_context_t* ctx)
    mbedtls_cipher_mode_t mbedtls_cipher_get_cipher_mode(
        const mbedtls_cipher_context_t* ctx)
    int mbedtls_cipher_get_iv_size(
        const mbedtls_cipher_context_t* ctx)
    mbedtls_cipher_type_t mbedtls_cipher_get_type(
        const mbedtls_cipher_context_t* ctx)
    const char* mbedtls_cipher_get_name(
        const mbedtls_cipher_context_t* ctx)
    int mbedtls_cipher_get_key_bitlen(
        const mbedtls_cipher_context_t* ctx)
    # mbedtls_cipher_get_operation
    int mbedtls_cipher_setkey(
        mbedtls_cipher_context_t* ctx,
        const unsigned char* key,
        int key_bitlen,
        const mbedtls_operation_t operation)
    int mbedtls_cipher_set_padding_mode(
        mbedtls_cipher_context_t* ctx,
        mbedtls_cipher_padding_t mode
    )
    int mbedtls_cipher_set_iv(
        mbedtls_cipher_context_t* ctx,
        const unsigned char *iv, size_t iv_len
    )
    int mbedtls_cipher_reset(mbedtls_cipher_context_t* ctx)
    int mbedtls_cipher_update_ad(
        mbedtls_cipher_context_t* ctx,
        const unsigned char *ad, size_t ad_len)
    int mbedtls_cipher_update(
        mbedtls_cipher_context_t* ctx,
        const unsigned char *input, size_t ilen,
        unsigned char *output, size_t *olen)
    int mbedtls_cipher_finish(
        mbedtls_cipher_context_t* ctx,
        unsigned char *output, size_t *olen)

    # mbedtls_cipher_write_tag
    # mbedtls_cipher_check_tag

    # int mbedtls_cipher_crypt(
    #     mbedtls_cipher_context_t* ctx,
    #     const unsigned char* iv, size_t iv_len,
    #     const unsigned char* input, size_t ilen,
    #     unsigned char* output, size_t* olen)

    int mbedtls_cipher_auth_encrypt(
        mbedtls_cipher_context_t* ctx,
        const unsigned char* iv, size_t iv_len,
        const unsigned char* ad, size_t ad_len,
        const unsigned char* input, size_t ilen,
        unsigned char* output, size_t* olen,
        unsigned char* tag, size_t tag_len)

    int mbedtls_cipher_auth_decrypt(
        mbedtls_cipher_context_t* ctx,
        const unsigned char* iv, size_t iv_len,
        const unsigned char* ad, size_t ad_len,
        const unsigned char* input, size_t ilen,
        unsigned char* output, size_t* olen,
        const unsigned char* tag, size_t tag_len)


cdef class _CipherBase:
    # Encapsulate two contexts to push the keys into mbedtls ASAP.
    cdef mbedtls_cipher_context_t _enc_ctx
    cdef mbedtls_cipher_context_t _dec_ctx


cdef class Cipher(_CipherBase):
    cdef _crypt(
        self,
        mbedtls_cipher_context_t *ctx,
        const unsigned char[:] input,
    )


cdef class AEADCipher(_CipherBase):
    cdef const unsigned char[:] _ad
    cdef const unsigned char[:] _iv
    cdef _aead_encrypt(self,
                const unsigned char[:] iv,
                const unsigned char[:] ad,
                const unsigned char[:] input)
    cdef _aead_decrypt(self,
                const unsigned char[:] iv,
                const unsigned char[:] ad,
                const unsigned char[:] input,
                const unsigned char[:] tag)



# SPDX-License-Identifier: MIT
# Copyright (c) 2016, Elaborated Networks GmbH
# Copyright (c) 2019, Mathias Laurin

"""Alleged River Cipher 4 cipher (ARC4 or ARCFOUR) designed in 1987
at RSA Security."""


from mbedtls.exceptions import TLSError

from . import _cipher

__all__ = ["block_size", "key_size", "new"]

block_size = 1
key_size = 16


def new(key, mode=None, iv=None):
    """Return a `Cipher` object that can perform ARC4 encryption and
    decryption.

    Alleged River Cipher 4 cipher (ARC4 or ARCFOUR) designed in 1987
    at RSA Security.

    Parameters:
        key (bytes or None): The key to encrypt decrypt.  If None,
            encryption and decryption are unavailable.
        mode (None): The feedback mode is ignored for ARC4.
        iv (None): ARC4 does not use IV.

    """
    if len(key) != key_size:
        raise TLSError(
            msg="key size must be %i bytes, got %i" % (key_size, len(key))
        )
    name = ("ARC4-%i" % (len(key) * 8)).encode("ascii")
    return _cipher.Cipher(name, key, mode, iv)



# SPDX-License-Identifier: MIT
# Copyright (c) 2019, Stephen Y.

"""Salsa20 and the closely related ChaCha are stream ciphers developed by Daniel J. Bernstein.

"""


from mbedtls.exceptions import TLSError

from . import _cipher

__all__ = ["block_size", "key_size", "new"]


block_size = 1
key_size = 32


def new(key, mode, iv=None, ad=None):
    """Return a `Cipher` object that can perform ChaCha20 encryption and
    decryption.

    ChaCha was created by Daniel Bernstein as a variant of
    its Salsa cipher https://cr.yp.to/chacha/chacha-20080128.pdf
    ChaCha20 is the variant with 20 rounds, that was also standardized
    in RFC 7539.

    Parameters:
        key (bytes or None): The key to encrypt decrypt.  If None,
            encryption and decryption are unavailable.
        mode (Mode): The mode of operation of the cipher.
        iv (bytes or None): The initialization vector (IV).  The IV is
            required for every mode but ECB and CTR where it is ignored.
            If not set, the IV is initialized to all 0, which should not
            be used for encryption.
        ad (bytes or None): The associated data for ChaCha/Poly mode.

    """
    if len(key) != 32:
        raise TLSError(msg="key size must 32 bytes, got %r" % len(key))
    if mode == _cipher.Mode.STREAM:
        assert ad is None
        cipher = _cipher.Cipher(b"CHACHA20", key, mode, iv)
    elif mode == _cipher.Mode.CHACHAPOLY:
        ad = b"" if ad is None else ad
        cipher = _cipher.AEADCipher(b"CHACHA20-POLY1305", key, mode, iv, ad)
    else:
        raise TLSError(msg="unsupported mode %r" % mode)
    return cipher



import datetime as dt
import pickle
from pathlib import Path

import certifi
import pytest

from mbedtls import hashlib
from mbedtls.pk import ECC, RSA
from mbedtls.x509 import *

CRL_PEM = """
-----BEGIN X509 CRL-----
MIIBqzCBlDANBgkqhkiG9w0BAQUFADA7MQswCQYDVQQGEwJOTDERMA8GA1UEChMI
UG9sYXJTU0wxGTAXBgNVBAMTEFBvbGFyU1NMIFRlc3QgQ0EXDTExMDIyMDEwMjI1
OVoXDTE5MTEyNTEwMjI1OVowKDASAgEBFw0xMTAyMTIxNDQ0MDdaMBICAQMXDTEx
MDIxMjE0NDQwN1owDQYJKoZIhvcNAQEFBQADggEBAJYuWdKPdblMVWCnxpMnchuL
dqWzK2BA0RelCaGjpxuwX3NmLDm+5hKja/DJxaRqTOf4RSC3kcX8CdIldsLO96dz
//wAQdFPDhy6AFT5vKTO8ItPHDb7qFOqFqpeJi5XN1yoZGTB1ei0mgD3xBaKbp6U
yCOZJSIFomt7piT4GcgWVHLUmpyHDDeodNhYPrN0jf2mr+ECd9fQJYdz1qm0Xx+Q
NbKXDiPRmPX0qVleCZSeSp1JAmU4GoCO+96qQUpjgll+6xWya3UNj61f9sh0Zzr7
5ug2LZo5uBM/LpNR1K3TLxNCcg7uUPTn9r143d7ivJhPl3tEJn4PXjv6mlLoOgU=
-----END X509 CRL-----
"""
# This CRL is from mbed TLS: `tests/data_files/crl.pem`.


@pytest.fixture(scope="module")
def now():
    return dt.datetime.utcnow().replace(microsecond=0)


@pytest.fixture
def issuer_key():
    issuer_key = RSA()
    issuer_key.generate(key_size=1024)
    return issuer_key


@pytest.fixture
def subject_key():
    subject_key = RSA()
    subject_key.generate(key_size=1024)
    return subject_key


class _X509Base:
    @pytest.fixture
    def x509(self):
        raise NotImplementedError

    @pytest.fixture
    def der(self, x509):
        return x509.to_DER()

    @pytest.fixture
    def pem(self, x509):
        return x509.to_PEM()


class _CommonTests(_X509Base):
    @pytest.mark.parametrize("repr_", (repr, str), ids=lambda f: f.__name__)
    def test_repr(self, repr_, x509):
        assert isinstance(repr_(x509), str)

    def test_pickle(self, x509):
        assert x509 == pickle.loads(pickle.dumps(x509))

    def test_hash(self, x509):
        assert isinstance(hash(x509), int)

    def test_from_buffer(self, x509, der):
        assert type(x509).from_buffer(der) == x509

    def test_from_file(self, x509, der, tmpdir):
        path = tmpdir.join("key.der")
        path.write_binary(der)
        assert type(x509).from_file(path) == x509

    def test_from_PEM_empty_buffer_raises_valueerror(self, x509):
        with pytest.raises(ValueError):
            type(x509).from_PEM("")

    def test_from_DER(self, x509, der):
        assert type(x509).from_DER(der) == x509

    def test_from_DER_empty_buffer_raises_valueerror(self, x509):
        with pytest.raises(ValueError):
            type(x509).from_DER(b"")

    def test_eq(self, x509):
        assert x509 == x509

    def test_eq_der(self, x509, der):
        assert x509 == der
        assert der == x509

    def test_eq_pem(self, x509, pem):
        assert x509 == pem
        assert pem == x509


class _CRTWikipediaBase(_X509Base):
    @pytest.fixture
    def x509(self):
        with (Path(__file__).parent / "ca/wikipedia.pem").open() as crt:
            return CRT(PEM_to_DER(crt.read()))

    @pytest.fixture
    def crt(self, x509):
        return x509


class TestCRTWikipediaBase(_CommonTests, _CRTWikipediaBase):
    pass


class TestCRTWikipediaAccessors(_CRTWikipediaBase):
    def test_issuer(self, crt):
        assert crt.issuer == ", ".join(
            (
                "C=US",
                "O=DigiCert Inc",
                "OU=www.digicert.com",
                "CN=DigiCert SHA2 High Assurance Server CA",
            )
        )

    def test_subject(self, crt):
        assert crt.subject == ", ".join(
            (
                "C=US",
                "ST=California",
                "L=San Francisco",
                "O=Wikimedia Foundation, Inc.",
                "CN=*.wikipedia.org",
            )
        )

    def test_subject_alternative_names(self, crt):
        assert "*.m.wikidata.org" in crt.subject_alternative_names
        assert len(crt.subject_alternative_names) == 41

    def test_key_usage(self, crt):
        assert crt.key_usage is KeyUsage.DIGITAL_SIGNATURE


class _CRTBase(_X509Base):
    @pytest.fixture
    def issuer(self):
        return "C=NL, O=PolarSSL, CN=PolarSSL Test CA"

    @pytest.fixture
    def subject(self):
        return "C=NL"

    @pytest.fixture
    def serial_number(self):
        return 0x1234567890

    @pytest.fixture
    def digestmod(self):
        return hashlib.sha256()

    @pytest.fixture
    def basic_constraints(self):
        return BasicConstraints(False, 0)

    @pytest.fixture
    def x509(
        self,
        now,
        issuer,
        issuer_key,
        subject,
        subject_key,
        serial_number,
        digestmod,
        basic_constraints,
    ):
        return CRT.new(
            not_before=now,
            not_after=now + dt.timedelta(days=90),
            issuer=issuer,
            issuer_key=issuer_key,
            subject=subject,
            subject_key=subject_key,
            serial_number=serial_number,
            digestmod=digestmod,
            basic_constraints=basic_constraints,
        )

    @pytest.fixture
    def crt(self, x509):
        return x509


class TestCRTBase(_CommonTests, _CRTBase):
    pass


class TestCRTAccessors(_CRTBase):
    def test_version(self, crt):
        assert crt.version == 3

    def test_not_before(self, crt, now):
        assert crt.not_before == now

    def test_not_after(self, crt, now):
        assert crt.not_after == now + dt.timedelta(days=90)

    def test_issuer(self, crt, issuer):
        assert crt.issuer == issuer

    def test_public_key(self, crt):
        pem = crt.subject_public_key.export_public_key(format="PEM")
        assert pem.startswith("-----BEGIN PUBLIC KEY-----\n")
        assert pem.rstrip("\0").endswith("-----END PUBLIC KEY-----\n")

    def test_subject(self, crt, subject):
        assert crt.subject == subject

    def test_serial_number(self, crt, serial_number):
        assert crt.serial_number == serial_number

    def test_revocation_bad_cast(self, der):
        crt = CRT.from_buffer(der)
        with pytest.raises(TypeError):
            crt.check_revocation(crt)

    def test_next(self):
        crt = CRT.from_file(certifi.where())
        with pytest.raises(StopIteration):
            while True:
                crt = next(crt)


class TestCRTMDAlg(_CRTBase):
    @pytest.fixture(params=[hashlib.sha1, hashlib.sha256])
    def digestmod(self, request):
        return request.param()

    def test_digestmod(self, crt, digestmod):
        assert crt.digestmod.name == digestmod.name


class TestCRTCAPath(_CRTBase):
    @pytest.fixture(params=[(True, 0), (True, 2), (False, 0), None])
    def basic_constraints(self, request):
        return request.param

    def test_ca(self, crt, basic_constraints):
        if basic_constraints is None:
            basic_constraints = BasicConstraints()
        assert crt.basic_constraints == basic_constraints


class _CSRBase(_X509Base):
    @pytest.fixture
    def subject(self):
        return "C=NL, O=PolarSSL, CN=PolarSSL Server 1"

    @pytest.fixture
    def x509(self, subject, subject_key):
        return CSR.new(subject_key, subject, hashlib.sha1())

    @pytest.fixture
    def csr(self, x509):
        return x509


class TestCSRBase(_CommonTests, _CSRBase):
    pass


class TestCSRAccessors(_CSRBase):
    def test_version(self, csr):
        assert csr.version == 1

    def test_subject(self, csr, subject):
        assert csr.subject == subject

    def test_subject_public_key(self, csr, subject_key):
        assert csr.subject_public_key == subject_key.export_public_key()


class _CRLBase(_X509Base):
    @pytest.fixture
    def x509(self):
        return CRL.from_PEM(CRL_PEM)

    @pytest.fixture
    def crl(self, x509):
        return x509


class TestCRLBase(_CommonTests, _CRLBase):
    pass


class TestCRLAccessors(_CRLBase):
    def test_tbs_certificate(self, crl):
        assert isinstance(crl.tbs_certificate, bytes)
        assert crl.tbs_certificate

    def test_signature_value(self, crl):
        assert isinstance(crl.signature_value, bytes)
        assert crl.signature_value

    def test_version(self, crl):
        assert crl.version == 1

    def test_issuer_name(self, crl):
        assert crl.issuer_name == "C=NL, O=PolarSSL, CN=PolarSSL Test CA"

    def test_this_update(self, crl):
        assert crl.this_update == dt.datetime(2011, 2, 20, 10, 22, 59)

    def test_next_update(self, crl):
        assert crl.next_update == dt.datetime(2019, 11, 25, 10, 22, 59)

    def test_revoked_certificates(self, crl):
        assert len(crl.revoked_certificates) == 2
        entry = crl.revoked_certificates[0]
        assert entry.revocation_date == dt.datetime(2011, 2, 12, 14, 44, 7)
        assert entry.serial == 1


class TestCRL(_CRLBase):
    @pytest.mark.skip("not implemented")
    def test_revocation_false(self, der):
        pass
        # crt = CRT.from_buffer(der)
        # crl = CRL.from_buffer(crl_der)
        # assert crt.check_revocation(crl) is False

    @pytest.mark.skip("not implemented")
    def test_crt_revocation_true(self, der):
        pass


class TestVerifyCertificateChain:
    @pytest.fixture
    def ca0_key(self):
        ca0_key = RSA()
        ca0_key.generate()
        return ca0_key

    @pytest.fixture
    def ca1_key(self):
        ca1_key = ECC()
        ca1_key.generate()
        return ca1_key

    @pytest.fixture
    def ee0_key(self):
        ee0_key = ECC()
        ee0_key.generate()
        return ee0_key

    @pytest.fixture
    def ca0_crt(self, ca0_key, now):
        ca0_csr = CSR.new(ca0_key, "CN=Trusted CA", hashlib.sha256())
        return CRT.selfsign(
            ca0_csr,
            ca0_key,
            not_before=now,
            not_after=now + dt.timedelta(days=90),
            serial_number=0x123456,
            basic_constraints=BasicConstraints(True, -1),
        )

    @pytest.fixture
    def ca1_crt(self, ca1_key, ca0_crt, ca0_key, now):
        ca1_csr = CSR.new(ca1_key, "CN=Intermediate CA", hashlib.sha256())
        return ca0_crt.sign(
            ca1_csr,
            ca0_key,
            now,
            now + dt.timedelta(days=90),
            0x234567,
            basic_constraints=BasicConstraints(True, 1),
        )

    @pytest.fixture
    def ee0_crt(self, ee0_key, ca1_crt, ca1_key, now):
        ee0_csr = CSR.new(ee0_key, "CN=End Entity", hashlib.sha256())
        return ca1_crt.sign(
            ee0_csr, ca1_key, now, now + dt.timedelta(days=90), 0x345678
        )

    def test_verify_chain(self, ca0_crt, ca1_crt, ee0_crt):
        assert all((ca1_crt.verify(ee0_crt), ca0_crt.verify(ca0_crt)))



"""Unit tests for mbedtls.pk."""

import numbers
import pickle

import pytest

import mbedtls
from mbedtls import hashlib
from mbedtls.exceptions import *
from mbedtls.mpi import MPI
from mbedtls.pk import *
from mbedtls.pk import ECPoint, _get_md_alg, _type_from_name

try:
    long
except NameError:
    long = int


def test_supported_curves():
    assert get_supported_curves()


def test_get_supported_ciphers():
    assert get_supported_ciphers()


@pytest.mark.parametrize(
    "md_algorithm",
    [vars(hashlib)[name] for name in hashlib.algorithms_available],
)
def test_digestmod_from_ctor(md_algorithm):
    assert callable(md_algorithm)
    algorithm = _get_md_alg(md_algorithm)
    assert isinstance(algorithm(), hashlib.Hash)


class TestECPoint:
    @pytest.fixture(params=[(MPI(1), MPI(2), MPI(3)), (1, 2, 3)])
    def xyz(self, request):
        return request.param

    @pytest.fixture
    def point(self, xyz):
        return ECPoint(*xyz)

    @pytest.mark.parametrize("repr_", (repr, str), ids=lambda f: f.__name__)
    def test_repr(self, repr_, point):
        assert isinstance(repr_(point), str)

    def test_pickle(self, point):
        assert point == pickle.loads(pickle.dumps(point))

    def test_hash(self, point):
        assert isinstance(hash(point), int)

    def test_accessors(self, point, xyz):
        x, y, z = xyz
        assert point.x == x
        assert point.y == y
        assert point.z == z

    def test_eq_point(self, point, xyz):
        assert (point == ECPoint(*xyz)) is True
        assert (point == ECPoint(0, 0, 0)) is False

    def test_eq_zero(self):
        zero = ECPoint(0, 0, 0)
        assert (zero == 1) is False
        assert (zero == 0) is True
        assert (zero == ECPoint(0, 0, 0)) is True

    def test_bool(self, point):
        assert bool(point) is True
        assert bool(ECPoint(0, 0, 0)) is False


class _TestCipherBase:
    @pytest.fixture
    def cipher(self):
        raise NotImplementedError

    @pytest.fixture
    def key(self, cipher):
        raise NotImplementedError

    @pytest.fixture
    def pub(self, cipher, key):
        return type(cipher).from_buffer(cipher.export_public_key())

    def test_pickle(self, cipher):
        assert cipher == pickle.loads(pickle.dumps(cipher))

    def test_hash(self, cipher):
        assert isinstance(hash(cipher), int)

    @pytest.mark.usefixtures("key")
    def test_cmp_eq(self, cipher):
        assert cipher == cipher

    @pytest.mark.parametrize("format", ["DER", "PEM"])
    @pytest.mark.usefixtures("key")
    def test_cmp_eq_prv(self, cipher, format):
        assert cipher == cipher.export_key(format)

    @pytest.mark.parametrize("format", ["DER", "PEM"])
    def test_cmp_eq_pub(self, pub, format):
        assert pub == pub.export_public_key(format)

    @pytest.mark.parametrize("invalid", [b"", "", b"\1\2\3", "123"])
    @pytest.mark.usefixtures("key")
    def test_cmp_neq(self, cipher, invalid):
        assert cipher != invalid

    def test_export_key_without_key(self, cipher):
        assert cipher.export_key("DER") == b""
        assert cipher.export_key("PEM") == ""

    def test_export_public_key_without_key(self, cipher):
        assert cipher.export_public_key("DER") == b""
        assert cipher.export_public_key("PEM") == ""

    @pytest.mark.usefixtures("key")
    def test_export_key_to_PEM(self, cipher):
        der = cipher.export_key("DER")
        other = type(cipher).from_DER(der)
        assert der != b""
        assert cipher == other

    @pytest.mark.usefixtures("key")
    def test_export_key_to_DER(self, cipher):
        pem = cipher.export_key("PEM")
        other = type(cipher).from_PEM(pem)
        assert pem != ""
        assert cipher == other

    @pytest.mark.usefixtures("key")
    def test_export_public_key_to_DER(self, cipher):
        der = cipher.export_public_key("DER")
        other = type(cipher).from_DER(der)
        assert der != b""
        assert other == cipher.export_public_key("DER")

    @pytest.mark.usefixtures("key")
    def test_export_public_key_to_PEM(self, cipher):
        pem = cipher.export_public_key("PEM")
        other = type(cipher).from_PEM(pem)
        assert pem != ""
        assert other == cipher.export_public_key("PEM")

    def test_generate(self, cipher, key):
        assert cipher.export_key()
        assert cipher.export_key() == key
        assert cipher.export_public_key()

    @pytest.mark.usefixtures("key")
    def test_type_accessor(self, cipher):
        assert cipher._type == _type_from_name(cipher.name)

    def test_key_size_accessor(self, cipher):
        assert cipher.key_size == 0

    @pytest.mark.usefixtures("key")
    def test_key_size_accessor_with_key(self, cipher):
        assert cipher.key_size != 0

    @pytest.mark.usefixtures("key")
    def test_check_pair(self, cipher):
        assert check_pair(cipher, cipher) is True

    @pytest.mark.parametrize(
        "digestmod",
        [_get_md_alg(name) for name in hashlib.algorithms_guaranteed],
        ids=lambda dm: dm().name,
    )
    def test_sign_without_key_returns_none(self, cipher, digestmod, randbytes):
        message = randbytes(4096)
        assert cipher.sign(message, digestmod) is None

    @pytest.mark.usefixtures("key")
    @pytest.mark.parametrize(
        "digestmod",
        [_get_md_alg(name) for name in hashlib.algorithms_guaranteed],
        ids=lambda dm: dm().name,
    )
    def test_sign_verify(self, cipher, digestmod, randbytes):
        msg = randbytes(4096)
        sig = cipher.sign(msg, digestmod)
        assert sig is not None
        assert cipher.verify(msg, sig, digestmod) is True
        assert cipher.verify(msg + b"\0", sig, digestmod) is False

    @pytest.mark.usefixtures("key")
    def test_import_public_key(self, cipher):
        pub = cipher.export_public_key()
        other = type(cipher).from_buffer(pub)
        assert not other.export_key()
        assert other.export_public_key()
        assert check_pair(cipher, other) is False  # Test private half.
        assert check_pair(other, cipher) is True  # Test public half.
        assert check_pair(other, other) is False
        assert cipher != other

    def test_import_private_key(self, cipher, key):
        other = type(cipher).from_buffer(key)
        assert other.export_key()
        assert other.export_public_key()
        assert check_pair(cipher, other) is True  # Test private half.
        assert check_pair(other, cipher) is True  # Test public half.
        assert check_pair(other, other) is True
        assert cipher == other


@pytest.mark.skipif(
    not mbedtls.has_feature("rsa"), reason="requires RSA support in libmbedtls"
)
class TestRSA(_TestCipherBase):
    @pytest.fixture
    def cipher(self):
        return RSA()

    @pytest.fixture
    def key(self, cipher):
        key_size = 1024
        return cipher.generate(key_size)

    @pytest.mark.usefixtures("key")
    def test_encrypt_decrypt(self, cipher, randbytes):
        msg = randbytes(cipher.key_size - 11)
        assert cipher.decrypt(cipher.encrypt(msg)) == msg


class TestECC(_TestCipherBase):
    @pytest.fixture(autouse=True, params=get_supported_curves())
    def cipher(self, request):
        curve = request.param
        return ECC(curve)

    @pytest.fixture
    def key(self, cipher):
        return cipher.generate()

    def test_export_key_to_num_without_key(self, cipher):
        assert cipher.export_key("NUM") == 0

    @pytest.mark.usefixtures("key")
    def test_export_key_to_num_with_key(self, cipher):
        assert cipher.export_key("NUM") != 0

    def test_export_public_key_to_point_without_key(self, cipher):
        assert cipher.export_public_key("POINT") == 0
        assert cipher.export_public_key("POINT") == ECPoint(0, 0, 0)

    @pytest.mark.usefixtures("key")
    def test_public_value_accessor(self, cipher):
        pub = cipher.export_public_key("POINT")
        assert isinstance(pub.x, numbers.Integral)
        assert isinstance(pub.y, numbers.Integral)
        assert isinstance(pub.z, numbers.Integral)
        assert pub.x not in (0, pub.y, pub.z)
        assert pub.y not in (0, pub.x, pub.z)
        assert pub.z in (0, 1)

    @pytest.mark.usefixtures("key")
    def test_private_value_accessor(self, cipher):
        prv = cipher.export_key("NUM")
        assert isinstance(prv, numbers.Integral)
        assert prv != 0


class TestECCtoECDH:
    @pytest.fixture(params=get_supported_curves())
    def curve(self, request):
        return request.param

    @pytest.fixture
    def ecp(self, curve):
        ecp = ECC(curve)
        ecp.generate()
        return ecp

    @pytest.fixture
    def srv(self, ecp):
        return ecp.to_ECDH_server()

    @pytest.fixture
    def cli(self, ecp):
        return ecp.to_ECDH_client()

    def test_exchange(self, srv, cli):
        cke = cli.generate()
        assert cli._has_public()

        srv.import_CKE(cke)
        assert srv._has_peers_public() is True

        srv_sec = srv.generate_secret()
        cli_sec = cli.generate_secret()
        assert srv_sec == cli_sec


class _TestDHBase:
    @pytest.fixture
    def modulus_size(self):
        return 64

    @pytest.fixture
    def generator_size(self):
        return 20

    @pytest.fixture
    def modulus(self, modulus_size):
        return MPI.prime(modulus_size)

    @pytest.fixture
    def generator(self, generator_size):
        return MPI.prime(generator_size)

    @pytest.fixture
    def dhentity(self, modulus, generator):
        raise NotImplementedError

    def test_pickle(self, dhentity):
        with pytest.raises(TypeError) as excinfo:
            pickle.dumps(dhentity)

        assert str(excinfo.value).startswith("cannot pickle")

    def test_modulus(self, dhentity, modulus):
        assert dhentity.modulus == modulus

    def test_generator(self, dhentity, generator):
        assert dhentity.generator == generator

    def test_key_size_accessor(self, dhentity):
        assert dhentity.key_size == 8

    def test_share_secret_accessor_default(self, dhentity):
        assert dhentity.shared_secret == 0


class TestDHServer(_TestDHBase):
    @pytest.fixture
    def dhentity(self, modulus, generator):
        return DHServer(modulus, generator)


class TestDHClient(_TestDHBase):
    @pytest.fixture
    def dhentity(self, modulus, generator):
        return DHClient(modulus, generator)


class TestDHExchange:
    @pytest.fixture
    def modulus_size(self):
        return 64

    @pytest.fixture
    def generator_size(self):
        return 20

    @pytest.fixture
    def cli(self, modulus_size, generator_size):
        return DHClient(MPI.prime(modulus_size), MPI.prime(generator_size))

    @pytest.fixture
    def srv(self, modulus_size, generator_size):
        return DHServer(MPI.prime(modulus_size), MPI.prime(generator_size))

    def test_exchange(self, srv, cli):
        ske = srv.generate()
        cli.import_SKE(ske)
        cke = cli.generate()
        srv.import_CKE(cke)

        srv_sec = srv.generate_secret()
        cli_sec = cli.generate_secret()
        assert srv_sec == cli_sec
        assert srv_sec == srv.shared_secret
        assert cli_sec == cli.shared_secret


class TestECDH:
    @pytest.fixture(params=get_supported_curves())
    def curve(self, request):
        return request.param

    @pytest.fixture
    def srv(self, curve):
        return ECDHServer(curve)

    @pytest.fixture
    def cli(self, curve):
        return ECDHClient(curve)

    def test_srv_pickle(self, srv):
        with pytest.raises(TypeError) as excinfo:
            pickle.dumps(srv)

        assert str(excinfo.value).startswith("cannot pickle")

    def test_cli_pickle(self, cli):
        with pytest.raises(TypeError) as excinfo:
            pickle.dumps(cli)

        assert str(excinfo.value).startswith("cannot pickle")

    def test_srv_key_accessors_without_key(self, srv):
        assert not srv._has_private()
        assert not srv._has_public()
        assert not srv._has_peers_public()
        assert srv.private_key == 0
        assert srv.public_key == 0
        assert srv.peers_public_key == 0
        assert srv.shared_secret == 0

    def test_cli_key_accessors_without_key(self, cli):
        assert not cli._has_private()
        assert not cli._has_public()
        assert not cli._has_peers_public()
        assert cli.private_key == 0
        assert cli.public_key == 0
        assert cli.peers_public_key == 0
        assert cli.shared_secret == 0

    def test_exchange(self, srv, cli):
        ske = srv.generate()
        assert srv._has_public()

        cli.import_SKE(ske)
        assert cli._has_peers_public() is True

        cke = cli.generate()
        assert cli._has_public()

        srv.import_CKE(cke)
        assert srv._has_peers_public() is True

        srv_sec = srv.generate_secret()
        cli_sec = cli.generate_secret()
        assert srv_sec == srv.shared_secret
        assert cli_sec == cli.shared_secret
        assert srv_sec == cli_sec
        assert srv.shared_secret == cli.shared_secret

    def test_generate_public(self, srv, cli):
        srv.generate()
        cli._private_key = srv.private_key
        assert cli.public_key != srv.public_key
        cli.generate_public_key()
        assert cli.public_key == srv.public_key


class TestECDHNaive:
    @pytest.fixture(params=[Curve.CURVE448, Curve.CURVE25519])
    def curve(self, request):
        return request.param

    @pytest.fixture
    def alice(self, curve):
        return ECDHNaive(curve)

    @pytest.fixture
    def bob(self, curve):
        return ECDHNaive(curve)

    def test_key_accessors_without_key(self, alice, bob):
        for peer in (alice, bob):
            assert not peer._has_private()
            assert not peer._has_public()
            assert peer.shared_secret == 0
            assert peer.private_key == 0
            assert peer.public_key == 0
            assert peer.peers_public_key == 0

    def test_exchange(self, alice, bob):
        alice_to_bob = alice.generate()
        assert alice._has_public()

        bob_to_alice = bob.generate()
        assert bob._has_public()

        assert alice.private_key != 0
        assert bob.private_key != 0
        assert alice.private_key != bob.private_key

        assert alice.public_key != 0
        assert bob.public_key != 0
        assert alice.public_key != bob.public_key

        alice.import_peers_public(bob_to_alice)
        assert alice._has_peers_public() is True
        assert alice.peers_public_key == bob.public_key

        bob.import_peers_public(alice_to_bob)
        assert bob._has_peers_public() is True
        assert bob.peers_public_key == alice.public_key

        alice_secret = alice.generate_secret()
        bob_secret = bob.generate_secret()
        assert alice_secret == bob_secret
        assert alice_secret == alice.shared_secret
        assert bob_secret == bob.shared_secret


class TestECDHNaiveAttacks:
    @pytest.fixture(params=[Curve.CURVE25519, Curve.CURVE448])
    def curve(self, request):
        return request.param

    @pytest.fixture
    def alice(self, curve):
        return ECDHNaive(curve)

    @pytest.fixture
    def bob(self, curve):
        return ECDHNaive(curve)

    @pytest.fixture
    def eve(self, curve):
        return ECDHNaive(curve)

    @pytest.fixture(autouse=True)
    def authenticate_alice_and_bob(self, alice, bob):
        alice_to_bob = alice.generate()
        bob_to_alice = bob.generate()
        alice.import_peers_public(bob_to_alice)
        bob.import_peers_public(alice_to_bob)
        alice_secret = alice.generate_secret()
        bob_secret = bob.generate_secret()

    def test_attacker_fails_with_public_keys(self, alice, bob, eve):
        eve._public_key = alice.public_key
        eve._peer_public_key = bob.public_key
        with pytest.raises(TLSError):
            eve.generate_secret()

    def test_attacker_succeeds_with_private_key(self, alice, bob, eve):
        eve._peer_public_key = bob.public_key
        eve._private_key = alice.private_key
        assert eve.generate_secret() == alice.shared_secret



# SPDX-license-identifier: MIT
# Copyright (c) 2019, Mathias Laurin

"""HMAC-based key derivation function (HKDF).

The HMAC-based extract-and-expand key derivation function specified
by RFC 5869.

"""


from libc.stdlib cimport malloc, free

cimport mbedtls._md as _hmac
cimport mbedtls.hkdf as _hkdf

import mbedtls.exceptions as _exc
import mbedtls.hmac as _hmac


__all__ = ("hkdf", "extract", "expand")


def hkdf(
    const unsigned char[:] key not None,
    length,
    const unsigned char[:] info not None,
    const unsigned char[:] salt=None,
    digestmod=None,
):
    """HMAC-based extract-and-expand key derivation function (HKDF).

    Arguments:
        key (bytes): The input keying material.
        length (int): The length of the output keying material in bytes.
        info (bytes): Additional context and application specific information.
        salt (bytes, optional): A non-secret random value.
        digestmod (hmac function, optional): The HMAC function to use for
            the extraction, defaults to SHA256.

    """
    if key.size == 0:
        key = b"\0"
    if digestmod is None:
        digestmod = _hmac.sha256
    cdef _hmac.Hmac hmac = digestmod(key)

    cdef const unsigned char *c_info
    if info.size == 0:
        c_info = NULL
    else:
        c_info = &info[0]
    cdef const unsigned char *c_salt
    if salt is None or salt.size == 0:
        c_salt = NULL
    else:
        c_salt = &salt[0]

    cdef unsigned char *okm = <unsigned char *>malloc(
        length * sizeof(unsigned char)
    )

    if not okm:
        raise MemoryError()
    try:
        _exc.check_error(_hkdf.mbedtls_hkdf(
            hmac._info,
            c_salt, 0 if salt is None else salt.size,
            &key[0], key.size,
            c_info, 0 if info is None else info.size,
            okm, length
        ))
        return okm[:length]
    finally:
        free(okm)


def extract(
    const unsigned char[:] key not None,
    const unsigned char[:] salt=None,
    digestmod=None,
):
    """Extract a fixed-length pseudorandom key.

    This function extracts a fixed-length pseudorandom key from
    its input keying material.

    This function should only be used if the security of it has been
    studied and established in that particular context (eg. TLS 1.3 key
    schedule).  For standard HKDF security guarantees use `hkdf` instead.

    Arguments:
        key (bytes): The input keying material.
        salt (bytes, optional): A non-secret random value.
        digestmod (hmac function, optional): The HMAC function to use for
            the extraction, defaults to SHA256.

    """
    if key.size == 0:
        key = b"\0"
    if digestmod is None:
        digestmod = _hmac.sha256
    cdef _hmac.Hmac hmac = digestmod(key)

    cdef const unsigned char *c_key
    if key.size == 0:
        c_key = NULL
    else:
        c_key = &key[0]
    cdef const unsigned char *c_salt
    if salt is None or salt.size == 0:
        c_salt = NULL
    else:
        c_salt = &salt[0]

    cdef unsigned char *prk = <unsigned char *>malloc(
        hmac.digest_size * sizeof(unsigned char)
    )
    if not prk:
        raise MemoryError()
    try:
        _exc.check_error(_hkdf.mbedtls_hkdf_extract(
            hmac._info,
            c_salt, 0 if salt is None else salt.size,
            c_key, key.size,
            prk
        ))
        return prk[:hmac.digest_size]
    finally:
        free(prk)


def expand(
    const unsigned char[:] prk not None,
    length,
    const unsigned char[:] info not None,
    digestmod=None,
):
    """Expand the pseudorandom key `prk` into additional pseudorandom keys.

    This function should only be used if the security of it has been
    studied and established in that particular context (eg. TLS 1.3 key
    schedule).  For standard HKDF security guarantees use `hkdf` instead.

    Arguments:
        prk (bytes): The pseudorandom key to expand, usually the
            output of `extract()`.
        length (int): The length of the output keying material in bytes.
        info (bytes): Additional context and application specific information.
        digestmod (hmac function, optional): The HMAC function to use for
            the extraction, defaults to SHA256.

    """
    if prk.size == 0:
        prk = b"\0"
    if digestmod is None:
        digestmod = _hmac.sha256
    cdef _hmac.Hmac hmac = digestmod(prk)

    cdef const unsigned char *c_info
    if info.size == 0:
        c_info = NULL
    else:
        c_info = &info[0]

    cdef unsigned char *okm = <unsigned char *>malloc(
        length * sizeof(unsigned char)
    )
    if not okm:
        raise MemoryError()
    try:
        _exc.check_error(_hkdf.mbedtls_hkdf_expand(
            hmac._info,
            &prk[0], prk.size,
            c_info, 0 if info is None else info.size,
            okm, length
        ))
        return okm[:length]
    finally:
        free(okm)



# SPDX-License-Identifier: MIT
# Copyright (c) 2019, Mathias Laurin

"""Declarations from `mbedtls/hkdf.h`."""


cimport mbedtls._md as _md


cdef extern from "mbedtls/hkdf.h" nogil:
    int mbedtls_hkdf(
        const _md.mbedtls_md_info_t *md,
        const unsigned char *salt, size_t salt_len,
        const unsigned char *ikm, size_t ikm_len,
        const unsigned char *info, size_t info_len,
        unsigned char *okm, size_t okm_len
    )

    int mbedtls_hkdf_extract(
        const _md.mbedtls_md_info_t *md,
        const unsigned char *salt, size_t salt_len,
        const unsigned char *ikm, size_t ikm_len,
        unsigned char *prk
    )

    int mbedtls_hkdf_expand(
        const _md.mbedtls_md_info_t *md,
        const unsigned char *prk, size_t prk_len,
        const unsigned char *info, size_t info_len,
        unsigned char *okm, size_t okm_len
    )
